from pyModeS import surv


def test_fs():
    assert surv.fs("2A00516D492B80")[0] == 2


def test_dr():
    assert surv.dr("2A00516D492B80")[0] == 0


def test_um():
    assert surv.um("200CBE4ED80137")[0] == 9
    assert surv.um("200CBE4ED80137")[1] == 1


def test_identity():
    assert surv.identity("2A00516D492B80") == "0356"


def test_altitude():
    assert surv.altitude("20001718029FCD") == 36000



from pyModeS import bds, commb
from pytest import approx

# from pyModeS import ehs, els    # deprecated


def test_bds20_callsign():
    assert bds.bds20.cs20("A000083E202CC371C31DE0AA1CCF") == "KLM1017_"
    assert bds.bds20.cs20("A0001993202422F2E37CE038738E") == "IBK2873_"

    assert commb.cs20("A000083E202CC371C31DE0AA1CCF") == "KLM1017_"
    assert commb.cs20("A0001993202422F2E37CE038738E") == "IBK2873_"


def test_bds40_functions():
    assert bds.bds40.selalt40mcp("A000029C85E42F313000007047D3") == 3008
    assert bds.bds40.selalt40fms("A000029C85E42F313000007047D3") == 3008
    assert bds.bds40.p40baro("A000029C85E42F313000007047D3") == 1020.0

    assert commb.selalt40mcp("A000029C85E42F313000007047D3") == 3008
    assert commb.selalt40fms("A000029C85E42F313000007047D3") == 3008
    assert commb.p40baro("A000029C85E42F313000007047D3") == 1020.0


def test_bds50_functions():
    msg1 = "A000139381951536E024D4CCF6B5"
    msg2 = "A0001691FFD263377FFCE02B2BF9"

    for module in [bds.bds50, commb]:
        assert module.roll50(msg1) == approx(2.1, 0.01)
        assert module.roll50(msg2) == approx(-0.35, 0.01)  # signed value
        assert module.trk50(msg1) == approx(114.258, 0.1)
        assert module.gs50(msg1) == 438
        assert module.rtrk50(msg1) == 0.125
        assert module.tas50(msg1) == 424


def test_bds60_functions():
    msg = "A00004128F39F91A7E27C46ADC21"

    for module in [bds.bds60, commb]:
        assert bds.bds60.hdg60(msg) == approx(42.71484)
        assert bds.bds60.ias60(msg) == 252
        assert bds.bds60.mach60(msg) == 0.42
        assert bds.bds60.vr60baro(msg) == -1920
        assert bds.bds60.vr60ins(msg) == -1920



import sys

import pytest
from pyModeS import bds

# this one fails on GitHub action for some unknown reason
# it looks successful on other Windows instances though
# TODO fix later
@pytest.mark.skipif(sys.platform == "win32", reason="GitHub Action")
def test_bds_infer():
    assert bds.infer("8D406B902015A678D4D220AA4BDA") == "BDS08"
    assert bds.infer("8FC8200A3AB8F5F893096B000000") == "BDS06"
    assert bds.infer("8D40058B58C901375147EFD09357") == "BDS05"
    assert bds.infer("8D485020994409940838175B284F") == "BDS09"

    assert bds.infer("A800178D10010080F50000D5893C") == "BDS10"
    assert bds.infer("A0000638FA81C10000000081A92F") == "BDS17"
    assert bds.infer("A0001838201584F23468207CDFA5") == "BDS20"
    assert bds.infer("A0001839CA3800315800007448D9") == "BDS40"
    assert bds.infer("A000139381951536E024D4CCF6B5") == "BDS50"
    assert bds.infer("A00004128F39F91A7E27C46ADC21") == "BDS60"


def test_bds_is50or60():
    assert bds.is50or60("A0001838201584F23468207CDFA5", 0, 0, 0) == None
    assert bds.is50or60("A8001EBCFFFB23286004A73F6A5B", 320, 250, 14000) == "BDS50"
    assert bds.is50or60("A8001EBCFE1B29287FDCA807BCFC", 320, 250, 14000) == "BDS50"


def test_surface_position():
    msg0 = "8FE48C033A9FA184B934E744C6FD"
    msg1 = "8FE48C033A9FA68F7C3D39B1C2F0"

    t0 = 1565608663102
    t1 = 1565608666214

    lat_ref = -23.4265448
    lon_ref = -46.4816258

    lat, lon = bds.bds06.surface_position(msg0, msg1, t0, t1, lat_ref, lon_ref)

    assert abs(lon_ref - lon) < 0.05



from pyModeS import allcall


def test_icao():
    assert allcall.icao("5D484FDEA248F5") == "484FDE"


def test_interrogator():
    assert allcall.interrogator("5D484FDEA248F5") == "SI6"


def test_capability():
    assert allcall.capability("5D484FDEA248F5")[0] == 5



from pyModeS import adsb
from pytest import approx

# === TEST ADS-B package ===


def test_adsb_icao():
    assert adsb.icao("8D406B902015A678D4D220AA4BDA") == "406B90"


def test_adsb_category():
    assert adsb.category("8D406B902015A678D4D220AA4BDA") == 0


def test_adsb_callsign():
    assert adsb.callsign("8D406B902015A678D4D220AA4BDA") == "EZY85MH_"


def test_adsb_position():
    pos = adsb.position(
        "8D40058B58C901375147EFD09357",
        "8D40058B58C904A87F402D3B8C59",
        1446332400,
        1446332405,
    )
    assert pos == (approx(49.81755, 0.001), approx(6.08442, 0.001))


def test_adsb_position_swap_odd_even():
    pos = adsb.position(
        "8D40058B58C904A87F402D3B8C59",
        "8D40058B58C901375147EFD09357",
        1446332405,
        1446332400,
    )
    assert pos == (approx(49.81755, 0.001), approx(6.08442, 0.001))


def test_adsb_position_with_ref():
    pos = adsb.position_with_ref("8D40058B58C901375147EFD09357", 49.0, 6.0)
    assert pos == (approx(49.82410, 0.001), approx(6.06785, 0.001))
    pos = adsb.position_with_ref("8FC8200A3AB8F5F893096B000000", -43.5, 172.5)
    assert pos == (approx(-43.48564, 0.001), approx(172.53942, 0.001))


def test_adsb_airborne_position_with_ref():
    pos = adsb.airborne_position_with_ref(
        "8D40058B58C901375147EFD09357", 49.0, 6.0
    )
    assert pos == (approx(49.82410, 0.001), approx(6.06785, 0.001))
    pos = adsb.airborne_position_with_ref(
        "8D40058B58C904A87F402D3B8C59", 49.0, 6.0
    )
    assert pos == (approx(49.81755, 0.001), approx(6.08442, 0.001))


def test_adsb_airborne_position_with_ref_numerical_challenge():
    lat_ref = 30.508474576271183 # Close to (360.0/59.0)*5
    lon_ref = 7.2*5.0+3e-15
    pos = adsb.airborne_position_with_ref(
        "8D06A15358BF17FF7D4A84B47B95", lat_ref, lon_ref
    )
    assert pos == (approx(30.50540, 0.001), approx(33.44787, 0.001))


def test_adsb_surface_position_with_ref():
    pos = adsb.surface_position_with_ref(
        "8FC8200A3AB8F5F893096B000000", -43.5, 172.5
    )
    assert pos == (approx(-43.48564, 0.001), approx(172.53942, 0.001))


def test_adsb_surface_position():
    pos = adsb.surface_position(
        "8CC8200A3AC8F009BCDEF2000000",
        "8FC8200A3AB8F5F893096B000000",
        0,
        2,
        -43.496,
        172.558,
    )
    assert pos == (approx(-43.48564, 0.001), approx(172.53942, 0.001))


def test_adsb_alt():
    assert adsb.altitude("8D40058B58C901375147EFD09357") == 39000


def test_adsb_velocity():
    vgs = adsb.velocity("8D485020994409940838175B284F")
    vas = adsb.velocity("8DA05F219B06B6AF189400CBC33F")
    vgs_surface = adsb.velocity("8FC8200A3AB8F5F893096B000000")
    assert vgs == (159, approx(182.88, 0.1), -832, "GS")
    assert vas == (375, approx(243.98, 0.1), -2304, "TAS")
    assert vgs_surface == (19, approx(42.2, 0.1), 0, "GS")
    assert adsb.altitude_diff("8D485020994409940838175B284F") == 550


def test_adsb_emergency():
    assert not adsb.is_emergency("8DA2C1B6E112B600000000760759")
    assert adsb.emergency_state("8DA2C1B6E112B600000000760759") == 0
    assert adsb.emergency_squawk("8DA2C1B6E112B600000000760759") == "6513"


def test_adsb_target_state_status():
    sel_alt = adsb.selected_altitude("8DA05629EA21485CBF3F8CADAEEB")
    assert sel_alt == (16992, "MCP/FCU")
    assert adsb.baro_pressure_setting("8DA05629EA21485CBF3F8CADAEEB") == 1012.8
    assert adsb.selected_heading("8DA05629EA21485CBF3F8CADAEEB") == approx(
        66.8, 0.1
    )
    assert adsb.autopilot("8DA05629EA21485CBF3F8CADAEEB") is True
    assert adsb.vnav_mode("8DA05629EA21485CBF3F8CADAEEB") is True
    assert adsb.altitude_hold_mode("8DA05629EA21485CBF3F8CADAEEB") is False
    assert adsb.approach_mode("8DA05629EA21485CBF3F8CADAEEB") is False
    assert adsb.tcas_operational("8DA05629EA21485CBF3F8CADAEEB") is True
    assert adsb.lnav_mode("8DA05629EA21485CBF3F8CADAEEB") is True


# def test_nic():
#     assert adsb.nic('8D3C70A390AB11F55B8C57F65FE6') == 0
#     assert adsb.nic('8DE1C9738A4A430B427D219C8225') == 1
#     assert adsb.nic('8D44058880B50006B1773DC2A7E9') == 2
#     assert adsb.nic('8D44058881B50006B1773DC2A7E9') == 3
#     assert adsb.nic('8D4AB42A78000640000000FA0D0A') == 4
#     assert adsb.nic('8D4405887099F5D9772F37F86CB6') == 5
#     assert adsb.nic('8D4841A86841528E72D9B472DAC2') == 6
#     assert adsb.nic('8D44057560B9760C0B840A51C89F') == 7
#     assert adsb.nic('8D40621D58C382D690C8AC2863A7') == 8
#     assert adsb.nic('8F48511C598D04F12CCF82451642') == 9
#     assert adsb.nic('8DA4D53A50DBF8C6330F3B35458F') == 10
#     assert adsb.nic('8D3C4ACF4859F1736F8E8ADF4D67') == 11
